/* QuakeMud, a silly mud using libhacknslash
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL collective
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>
#include <stdlib.h>

#include <hacknslash/misc.h>
#include <hacknslash/colors.h>

#include "commands.h"
#include "game.h"
#include "weapons.h"
#include "rooms.h"

#define HELP_DIR INSTALLDIR "/help"

static void
cmd_move (Player_t *p, const gchar *args, gpointer data)
{
  room_player_move (p, GPOINTER_TO_INT(data));
  player_send_prompt (p);
}

static void
cmd_attack (Player_t *p, const gchar *args, gpointer data)
{
  Player_t *t = NULL;
  if (args && strlen (args) > 0)
    {
      t = room_player_find (p->room, args);
      if (!t)
	{
	  t = player_find (args, TRUE);
	  if (!t)
	    {
	      player_send (p, COL_BAD, "Couldn't find anyone with that name.");
	      player_send_prompt (p);
	      return;
	    }
	  else if (!strcasecmp (p->team, t->team))
	    {
	      player_sendf (p, COL_BAD, "`%s' is on the same team as you!", t->name);
	      player_send_prompt (p);
	      return;
	    }
	}
    }
  if (t)
    {
      if (t == p->attacking)
	{
	  player_sendf (p, COL_BAD, "Already attacking player `%s'", t->name);
	}
      else if (t != p)
	{
	  player_sendf (p, COL_GOOD, "Attacking player `%s'", t->name);
	  player_set_attack (p, t);
	}
    }
  else
    {
      if (p->attacking)
	{
	  player_sendf (p, COL_GOOD, "No longer attacking `%s'", p->attacking->name);
	  player_set_attack (p, NULL);
	}
    }
}

static void
cmd_use (Player_t *p, const gchar *args, gpointer data)
{
  gint w = GPOINTER_TO_INT(data);
  if (w >= 0 && w < NUM_WEAPONS)
    {
      if (w == p->weapon)
	player_sendf (p, COL_GOOD, "You are already using the %s", weapons[w].name);
      else if (player_has_weapon (p, w))
	{
	  if (weapons[w].ammo >= 0 &&
	      weapons[w].ammo_use > p->ammo[ weapons[w].ammo ])
	    player_send (p, COL_BAD, "You don't have enough ammo.");
	  else
	    {
	      player_sendf (p, COL_GOOD, "Switching to %s.", weapons[w].name);
	      p->weapon = w;
	    }
	}
      else
	player_sendf (p, COL_BAD, "You don't have the %s.", weapons[w].name);
    }
}

/* */
static void
cmd_help (Player_t *p, gchar * args, gpointer data)
{
  int len;
  if (args && 0 < (len = strlen (args)))
    {
      QCmd_t *cmd = (QCmd_t *) cmd_find (&game_cmd_list, args, len);
      if (cmd && cmd->help >= 0)
	{
	  gchar file[64];
	  snprintf (file, 64, HELP_DIR "/%s", cmd->help);
	  player_spool (p, file);
	  return;
	}
    }
  player_spool (p, HELP_DIR "/help");
}

static void
cmd_inventory (Player_t *p, const gchar *args, gpointer data)
{
  player_send_stats (p);
}

static void
cmd_look (Player_t *p, const gchar *args, gpointer data)
{
  room_look (p->room, p);
}

static void
cmd_quit (Player_t *p, const gchar *args, gpointer data)
{
  player_send (p, COL_BAD, "Coward! Come back! Come back and fIgHT!\n");
  server_disconnect ((Sock_t *) p);
}

static void
cmd_scores (Player_t *p, const gchar *args, gpointer data)
{
  player_send_scores (p);
}

static void
cmd_who (Player_t *p, const char *args, gpointer data)
{
  player_send_players (p);
}

static void
cmd_say (Player_t *p, const gchar *args, gpointer data)
{
  if (args && strlen (args) > 0)
    {
      if (p->say > SAY_COST)
	{
	  p->say -= SAY_COST;
	  room_sendf (p->room, COL_LTCYAN, "\n`%s' said \"%s\".", p->name, args);
	  players_send_prompts ();
	}
      else
	{
	  player_send (p, COL_BAD, "Stop mucking around and kill someone.");
	  p->tell_me_when_i_can_talk = TRUE;
	  player_send_prompt (p);
	}
    }
}

static void
cmd_shout (Player_t *p, const gchar *args, gpointer data)
{
  if (args && strlen (args) > 0)
    {
      if (p->say > SHOUT_COST)
	{
	  p->say -= SHOUT_COST;
	  players_sendf (COL_LTCYAN, "\n`%s' shouted \"%s\".", p->name, args);
	  players_send_prompts ();
	}
      else
	{
	  /* tell the player something here, i can't figure out what though */
	}
    }
}

static gchar * emote_text[ 3 ] =
{
  "\n`%s' waves.",
  "\n`%s' flips everyone the bird.",
  "\n`%s' salutes."
};

static void
cmd_emote (Player_t *p, const gchar *args, gpointer data)
{
  gint which = GPOINTER_TO_INT(data); 
  if (p->say > SAY_COST)
    {
      p->say -= SAY_COST;
      room_sendf (p->room, COL_LTGREY, emote_text[ which ], p->name);
      players_send_prompts ();
    }
  else
    {
      p->tell_me_when_i_can_talk = TRUE;
    }
}

static void
cmd_point (Player_t *p, const gchar *args, gpointer data)
{
  gint dir;
  gint arg_len = 0;
  if (p->say > SAY_COST)
    {
      p->say -= SAY_COST;
      if (args)
	arg_len = strlen (args);
      if (arg_len)
	{
	  for (dir = 0; dir < NUM_EXITS; dir++)
	    {
	      if (!strncasecmp (args, exit_dirs[dir], arg_len))
		room_sendf (p->room, COL_LTGREY, "\n`%s' points %s.", p->name, exit_dirs[ dir ]);
	    }
	}
      else
	dir = NUM_EXITS;
      if (dir == NUM_EXITS)
	room_sendf (p->room, COL_LTGREY, "\n`%s' points at nothing in particular.", p->name);
      players_send_prompts ();
    }
  else
    {
      p->tell_me_when_i_can_talk = TRUE;
    }
}

void
cmd_peace (Player_t *p, const gchar *args, gpointer data)
{
  if (p->peace_mode)
    {
      player_send (p, COL_GOOD, "Alright!, lets kill some people!");
      p->peace_mode = FALSE;
    }
  else
    {
      player_send (p, COL_GOOD, "Entering peace-mode, you tree-hugging hippy!");
      p->peace_mode = TRUE;
    }
}

static void
cmd_suicide (Player_t *p, const gchar *args, gpointer data)
{
  player_suicide (p);
  players_send_prompts ();
}



static void
_input_rpassword (Player_t *p, gchar *line)
{
  server_mute ((Sock_t *) p, FALSE);
  server_echo ((Sock_t *) p, FALSE);
  if (!misc_strcmp (line, game_rpassword))
    {
      player_send (p, COL_GOOD, "\r\nPassword correct.  You now have remote admin prilileges");
      p->admin = TRUE;
    }
  else
    player_send (p, COL_BAD, "\r\nRemote admin password incorrect");
  p->handler = NULL;
}

#define _access_denied(P) \
  player_send ((P), \
	       COL_BAD, \
	       "\nYou need to be an administrator to use that command" \
	       "\nUse the rlogin command to authenticate as an administrator"); \

static void
cmd_rlogin (Player_t *p, const gchar *args, gpointer data)
{
  /* authenticate `p' as a remote admin */
  if (!game_rpassword)
    {
      player_send (p, COL_BAD, "Remote admin has been disabled");
    }
  else
    {
      player_send (p, -1, "Enter remote admin password\r\n> ");
      p->handler = _input_rpassword;
      server_echo ((Sock_t *) p, TRUE);
      server_mute ((Sock_t *) p, TRUE);
    }
}

static void
cmd_map (Player_t *p, const gchar *args, gpointer data)
{
  if (p->admin)
    {
      /* find map `args' in maps directory and load it.
       * map rotation is left unchanged
       */
      if (args)
	{
	  Block_t *info;
	  if (!(info = game_map_info (args)))
	    player_sendf (p, COL_BAD, "Map \"%s\" could not be found.", args);
	  else
	    {
	      block_free (info);
	      game_map_change (args);
	      players_send_prompts ();
	    }
	}
    }
  else
    _access_denied (p);
}

static void
cmd_maps (Player_t *p, const gchar *args, gpointer data)
{
  game_maps_list (p);
}

static void
cmd_mapchange (Player_t *p, const gchar *args, gpointer data)
{
  if (p->admin)
    {
      game_map_rotate ();
      players_send_prompts ();
    }
  else
    _access_denied (p);
}

static void
cmd_reload (Player_t *p, const gchar *args, gpointer data)
{
  if (p->admin)
    {
      game_broadcast (COL_EVENT, "\nReloading config...");
      game_config_load ();
      players_send_prompts ();
    }
  else
    _access_denied (p);
}

static void
cmd_timelimit (Player_t *p, const gchar *args, gpointer data)
{
  if (p->admin)
    {
      int newlimit = 0;
      char *endptr;
      if (args)
	{
	  endptr = NULL;
	  newlimit = strtol (args, &endptr, 10);
	  if (endptr)
	    {
	      if (newlimit < 0)
		newlimit = 0;
	      if ((newlimit*60) != game_time_limit)
		{
		  game_time_limit = (newlimit*60);
		  game_broadcastf (COL_EVENT,
				   "\n`%s' set the time limit to %d minutes",
				   p->name,
				   newlimit );
		  players_send_prompts();
		}
	      return;
	    }
	}
      
      player_sendf (p, -1, "\nThe current time limit is %d minutes\n", game_time_limit);
    }
  else
    _access_denied (p);
}

static void
cmd_fraglimit (Player_t *p, const gchar *args, gpointer data)
{
  if (p->admin)
    {
      int newlimit = 0;
      char *endptr;
      if (args)
	{
	  endptr = NULL;
	  newlimit = strtol (args, &endptr, 10);
	  if (endptr)
	    {
	      if (newlimit < 0)
		newlimit = 0;
	      if (newlimit != game_frag_limit)
		{
		  game_frag_limit = newlimit;
		  game_broadcastf (COL_EVENT,
				   "\n`%s' set the frag limit to %d",
				   p->name,
				   newlimit );
		  players_send_prompts ();
		  return;
		}
	    }
	}
      
      player_sendf (p, -1, "\nThe current frag limit is %d\n", game_frag_limit);
    }
  else
    _access_denied (p);
}

static void
cmd_stop (Player_t *p, const gchar *args, gpointer data)
{
  if (p->admin)
    {
      game_broadcastf (COL_EVENT, "\n`%s' stopped the game.\n", p->name);
      exit (EXIT_SUCCESS);
    }
  else
    _access_denied (p);
}

static void
cmd_spawn (Player_t *p, const gchar *args, gpointer data)
{
  if (p->state == PLAYER_STATE_LIMBO)
    player_spawn (p);
}



/* commands that can be used while in limbo or in game */
QCmd_t limbo_commands[] =
{
  {{"spawn",     (CmdFunc_t)cmd_spawn, NULL}, NULL},
  {{"respawn",   (CmdFunc_t)cmd_spawn, NULL}, NULL},

  {{"shout",     (CmdFunc_t)cmd_shout, NULL}, "say"},

  {{"scores",    (CmdFunc_t)cmd_scores, NULL}, "scores"},
  {{"help",      (CmdFunc_t)cmd_help, NULL}, NULL},
  {{"?",         (CmdFunc_t)cmd_help, NULL}, NULL},
  {{"who",       (CmdFunc_t)cmd_who, NULL}, "who"},

  {{"rlogin",    (CmdFunc_t)cmd_rlogin, NULL}, "rlogin"},
  {{"map",       (CmdFunc_t)cmd_map, NULL}, "map"},
  {{"mapchange", (CmdFunc_t)cmd_mapchange, NULL}, "mapchange"},
  {{"timelimit", (CmdFunc_t)cmd_timelimit, NULL}, "timelimit"},
  {{"fraglimit", (CmdFunc_t)cmd_fraglimit, NULL}, "fraglimit"},
  {{"reload",    (CmdFunc_t)cmd_reload, NULL}, "reload"},
  {{"stop",      (CmdFunc_t)cmd_stop, NULL}, "stop"},

  {{"maps",      (CmdFunc_t)cmd_maps, NULL}, "maps"},

  {{"quit",      (CmdFunc_t)cmd_quit, NULL}, "quit"},
  
  {{NULL,        NULL, NULL}, NULL}
};

/* commands only used in game */
QCmd_t game_commands[] = {
  {{"north",     (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_NORTH )}, "move"},
  {{"south",     (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_SOUTH )}, "move"},
  {{"west",      (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_WEST )}, "move"},
  {{"east",      (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_EAST )}, "move"},
  {{"up",        (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_UP )}, "move"},
  {{"down",      (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_DOWN )}, "move"},
  {{"teleport",  (CmdFunc_t)cmd_move, GINT_TO_POINTER( EXIT_TELEPORT )}, "move"},
  
  {{"1",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_AXE )}, NULL},
  {{"2",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_SHOTGUN )}, NULL},
  {{"3",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_SSHOTGUN )}, NULL},
  {{"4",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_NAILGUN )}, NULL},
  {{"5",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_SNAILGUN )}, NULL},
  {{"7",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_RLAUNCHER )}, NULL},
  {{"6",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_GLAUNCHER )}, NULL},
  {{"8",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_ZAPGUN )}, NULL},
  {{"9",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_RAILGUN )}, NULL},
  {{"0",         (CmdFunc_t)cmd_use, GINT_TO_POINTER( WEAPON_BFG )}, NULL},

  {{"look",      (CmdFunc_t)cmd_look, NULL }, "look"},  
  {{"inventory", (CmdFunc_t)cmd_inventory, NULL }, "inventory"},
  {{"kill",      (CmdFunc_t)cmd_attack, NULL }, "attack"},
  {{"attack",    (CmdFunc_t)cmd_attack, NULL }, "attack"},
  {{"wave",      (CmdFunc_t)cmd_emote, GINT_TO_POINTER( EMOTE_WAVE )}, "emote"},
  {{"flipoff",   (CmdFunc_t)cmd_emote, GINT_TO_POINTER( EMOTE_FLIPOFF )}, "emote"},
  {{"salute",    (CmdFunc_t)cmd_emote, GINT_TO_POINTER( EMOTE_SALUTE )}, "emote"},
  {{"point",     (CmdFunc_t)cmd_point, NULL }, "emote"},
  {{"peace",     (CmdFunc_t)cmd_peace, NULL }, "peace"},

  {{"say",       (CmdFunc_t)cmd_say, NULL}, "say"},
  {{"'",         (CmdFunc_t)cmd_say, NULL}, "say"},

  /* don't do it man, it's not worth it */
  {{"suicide",   (CmdFunc_t)cmd_suicide, NULL}, "suicide"},

  {{NULL,        NULL, NULL}, NULL}
};

