/* QuakeMud, a silly mud using libhacknslash
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL collective
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>

#include <hacknslash/misc.h>
#include <hacknslash/colors.h>

#include "items.h"
#include "game.h"

Item_t items[] =
{
  /* ammo */
  { "box of shells",  0, 0, 0, 0, { 50,  0, 0, 0,  0 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },
  { "box of nails",   0, 0, 0, 0, {  0, 50, 0, 0,  0 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },
  { "box of rockets", 0, 0, 0, 0, {  0,  0, 5, 0,  0 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },
  { "battery",        0, 0, 0, 0, {  0,  0, 0, 25, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },
  { "box of slugs",   0, 0, 0, 0, {  0,  0, 0, 0,  5 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },

  /* weapons */
  { "super shotgun",    0, 0, (1 << WEAPON_SSHOTGUN), 0, { 20, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "nailgun",          0, 0, (1 << WEAPON_NAILGUN), 0, { 0, 20, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "super nailgun",    0, 0, (1 << WEAPON_SNAILGUN), 0, { 0, 20, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "rocket launcher",  0, 0, (1 << WEAPON_RLAUNCHER), 0, { 0, 0, 5, 0, 0 },
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "grenade launcher", 0, 0, (1 << WEAPON_GLAUNCHER), 0, { 0, 0, 5, 0, 0 },
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "zapgun",           0, 0, (1 << WEAPON_ZAPGUN), 0, { 0, 0, 0, 25, 0 },
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "railgun",          0, 0, (1 << WEAPON_RAILGUN), 0, { 0, 0, 0, 0, 5 },
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },

  { "BFG",              0, 0, (1 << WEAPON_BFG), 0, { 0, 0, 0, 50, 0 },
    { 0, 0, 0, 0, 0, 0 }, 120, 0, 0, FALSE },
  
  /* health */
  { "health",     30,  0, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },
  { "adrenalin",  100, 0, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 60, 0, 0, FALSE },
  { "megahealth", 200, 0, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },

  /* armors */
  { "green armor",  0, 100, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },
  { "yellow armor", 0, 150, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },
  { "red armor",    0, 200, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },

  /* powerups */
  { "quad damage",     0, 0, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 30, 0, 0, 0, 0, 0 }, 180, 0, 0, TRUE },
  { "pentagram",       0, 0, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 30, 0, 0, 0, 0 }, 180, 0, 0, TRUE },
  { "ring of shadows", 0, 0, 0, 0, { 0, 0, 0, 0, 0 }, 
    { 0, 0, 30, 0, 0, 0 }, 180, 0, 0, TRUE },
  { "regeneration",    0, 0, 0, 0, { 0, 0, 0, 0, 0 },
    { 0, 0, 0, 30, 0, 0 }, 180, 0, 0, TRUE },
  { "haste",           0, 0, 0, 0, { 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 30, 0 }, 180, 0, 0, TRUE },
  { "environment suit", 0, 0, 0, 0, { 0, 0, 0, 0, 0 },
    { 0, 0, 0, 0, 0, 30 }, 180, 0, 0, TRUE },
    
  /* keys */
  { "iron skull-key",   0, 0, 0, (1 << KEY_IRON),   { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },
  { "silver skull-key", 0, 0, 0, (1 << KEY_SILVER), { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },
  { "gold skull-key",   0, 0, 0, (1 << KEY_GOLD),   { 0, 0, 0, 0, 0 }, 
    { 0, 0, 0, 0, 0, 0 }, 180, 0, 0, FALSE },

  { NULL, 0, 0, 0, 0, { 0, 0, 0, 0, 0 }, { 0, 0, 0, 0, 0 }, 0, 0, 0, FALSE }
};

/* create a new item of type `name' */
Item_t *
item_new (gchar *name)
{
  gint i;
  gint len;
  if (!name || 0 == (len = strlen (name)))
    return NULL;
  for (i = 0; items[i].name; i++)
    {
      if (!strncasecmp (name, items[i].name, len))
	{
	  Item_t *item;
	  item = g_new (Item_t, 1);
	  memcpy (item, &(items[i]), sizeof (Item_t));
	  item->name = g_strdup (items[i].name);
	  return item;
	}
    }
  return NULL;
}

Item_t *
item_alloc ()
{
  Item_t *item;
  item = g_new0 (Item_t, 1);
  return item;
}

void
item_free (Item_t *item)
{
  g_free (item->name);
  g_free (item);
}

/* ick */
gboolean
item_pickup (Player_t *p, Item_t *item)
{
  gboolean get = FALSE;
  gint i;

  if (item->spawn_time)
    return FALSE;

  if (item->weapons)
    {
      if ((p->weapons & item->weapons) != item->weapons)
	{
	  p->weapons |= item->weapons;
	  get = TRUE;
	}
      else if (!(item->always_pickup))
	/* can't pickup weapons we already have unless they're in backpacks */
	return FALSE;
    }

  if (item->health)
    {
      if (item->health <= 100)
	{
	  if (p->health < 100)
	    {
	      p->health = MIN(100, p->health + item->health);
	      get = TRUE;
	    }
	}
      else
	{
	  if (p->health < 200)
	    {
	      p->health = MIN(200, p->health + item->health);
	      get = TRUE;
	    }
	}
    }

  if (item->armor)
    {
      if (p->armor < item->armor)
	{
	  p->armor = item->armor;
	  get = TRUE;
	} 

    }

  if (item->keys)
    {
      if ((p->keys & item->keys) != item->keys)
	{
	  p->keys |= item->keys;
	  get = TRUE;
	}
    }

  for (i = 0; i < NUM_AMMO_TYPES; i++)
    {
      if (item->ammo[i] && p->ammo[i] < ammo_max_carry[i])
	{
	  p->ammo[i] += item->ammo[i];
	  get = TRUE;
	}
    }

  for (i = 0; i < NUM_POWERUPS; i++)
    {
      if (item->powerups[i])
	{
	  p->powerup[i] += item->powerups[i];
	  get = TRUE;
	}
    }

  if (get || item->always_pickup)
    {
      player_sendf (p, COL_EVENT, "\nYou got the %s.", item->name);
      /* FIXME: tell the room that the player got the item */
      return TRUE;
    }

  return FALSE;
}


