/* QuakeMud, a silly mud using libhacknslash
 * Copyright (C) 2000 David Wynn (aka kismet), of the OSOAL collective
 * send abuse to:    david@osoal.org.nz
 * http requests to: web.osoal.org.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ROOMS_H__
#define __ROOMS_H__

#include <glib.h>
#include <time.h>

#include <hacknslash/block.h>

typedef struct ExitClass ExitClass_t;
typedef struct Exit      Exit_t;

typedef struct RoomClass RoomClass_t;
typedef struct Room      Room_t;

#include "weapons.h"
#include "players.h"
#include "items.h"

/* */
enum
{
  EXIT_NORTH = 0,
  EXIT_SOUTH,
  EXIT_EAST,
  EXIT_WEST,
  EXIT_UP,
  EXIT_DOWN,
  EXIT_TELEPORT,

  NUM_EXITS
};

enum
{
  KEY_IRON = 0,
  KEY_SILVER,
  KEY_GOLD,

  NUM_KEYS
};

/* eg, "north", "south", "east",... */
extern const gchar *exit_dirs[ NUM_EXITS ];

/* */
extern const gint   opp_dirs[ NUM_EXITS ];

/* eg, "iron skull-key", "silver skull-key",... */
extern const gchar *key_names[ NUM_KEYS ];

struct ExitClass
{
  ObjectClass_t parent;
};

ExitClass_t *exit_class ();
BlockDef_t  *exit_block_def ();

struct Exit
{
  Object_t parent;

  gchar *name;
  gchar *descript;

  gint dir; /* EXIT_? */
  gint key; /* which key is required to go through this exit */
  GSList *rooms; /* possible destinations */

  gboolean secret; /* if true, exit does not show up in exit list */
};



/* */
struct RoomClass
{
  ObjectClass_t parent;
};

RoomClass_t *room_class ();
BlockDef_t  *room_block_def ();

/* room events */
typedef struct RoomEventPlayerMove RoomEventPlayerMove_t;
struct RoomEventPlayerMove
{
  ObjectEvent_t parent;
  Player_t *player;
  gint dir;    /* or -1 for spawn */
  gboolean in; /* if true, player moved into room, else moved out */
};

enum {
  ROOM_EVENT_THINK = 0,
  ROOM_EVENT_PLAYER_MOVE,

  NUM_ROOM_EVENTS
};

guint room_event_type( guint virtual_type );

/* */
enum {
  ROOM_FLAG_NOKILL = 0,
  ROOM_FLAG_WATER,
  ROOM_FLAG_SLIME,
  ROOM_FLAG_LAVA,

  NUM_ROOM_FLAGS
};

struct Room {
  Object_t parent;

  gboolean player_spawn; /* if true, this room has a player-spawn point */
  gchar   *name;
  gchar   *descript;

  GSList  *players;      /* players in this room */
  gint     num_players;

  GSList  *items;        /* items in this room */
  gint     num_items;
  
  guint    flags;        /* a bit mask of (1 << ROOM_FLAG_*) */
};

/* basic room database functions */
void    room_db_init ();
void    room_db_load (gchar *dir); /* load rooms from database in `dir' */
void    room_db_free ();

/* get the room `id' from the loaded room database */
Room_t *room_db_get  (guint  id);



/* make all rooms think (spawn items, do combat etc...) */
void    rooms_think  ();

/* return a random player-spawn room */
Room_t *room_spawn   ();

/* room functions */
void    room_player_remove (Room_t *room, Player_t *player);

void    room_player_add    (Room_t *room, Player_t *player);

/* try and move `player' through the exit `dir' */
void    room_player_move   (Player_t *player, gint dir);

void    room_item_add      (Room_t *room, Item_t *item);

/* send a detailed description of `room' to `player' */
void    room_look          (Room_t *room, Player_t *player);

/* send lists of contents of `room' to `player' */
void    room_items_list    (Room_t *room, Player_t *player);
void    room_players_list  (Room_t *room, Player_t *player);
void    room_exits_list    (Room_t *room, Player_t *player);

/* send to players in `room' */
void    room_send  (Room_t *room, gint col, gchar *text);
void    room_sendf (Room_t *room, gint col, gchar *format,...);

/* get random player in room */
Player_t *room_rand_player        (Room_t *room);
Player_t *room_rand_player_except (Room_t *room, gchar *team);

/* find player with name that matches the partial name `name' */
Player_t *room_player_find        (Room_t *room, gchar *name);

#endif /* __ROOMS_H__ */
